/*
 * stm.h
 *
 *  Created on: 15.12.2015
 *      Author: Renate Schultes
 */

#ifndef STM_H_
#define STM_H_
#include "stdint.h"

#define STM0_BASE    ((STM_t*)0xF0000000)
#define STM1_BASE    ((STM_t*)0xF0000100)
#define STM2_BASE    ((STM_t*)0xF0000200)

#define SYSTIME_ISR0_PRIO 1
#define SYSTIME_ISR1_PRIO 2

#define SRC_STM0SR0    (*((volatile uint32_t*) (0xF0038490+(0*8))))
#define SRC_STM0SR1    (*((volatile uint32_t*) (0xF0038494+(0*8))))
#define SRC_STM1SR0    (*((volatile uint32_t*) (0xF0038490+(1*8))))
#define SRC_STM1SR1    (*((volatile uint32_t*) (0xF0038494+(1*8))))
#define SRC_STM2SR0    (*((volatile uint32_t*) (0xF0038490+(2*8))))
#define SRC_STM2SR1    (*((volatile uint32_t*) (0xF0038494+(2*8))))

enum
{
	CLEAR       = 1,
	MSIZE_1     = 0x00000000,
	MSIZE_28    = 0x0000001B,
	MSIZE_32    = 0x0000001F,
	MSTART_0    = 0x00000000,
	MSTART_8    = 0x00000800,
	MSTART_31   = 0x00001F00,
};

enum
{
	CMP0EN      = (1<<0),
	CMP0OS_IR0  = (0<<2),
	CMP0OS_IR1  = (1<<2),
	CMP1EN      = (1<<4),
	CMP1OS_IR0  = (0<<6),
	CMP1OS_IR1  = (1<<6),
};

enum
{
	DISABLE = 0,
	ENABLE = 1,
};

typedef struct stm
{
    volatile unsigned int CLC;            // Clock Control Register
    volatile unsigned int RESERVED0;      // reserved (0x04)
    volatile unsigned int ID;             // ASCLIN Identification Register
    volatile unsigned int RESERVED1;      // reserved (0x0C)
    volatile unsigned int TIM0; 	      // STM[31:0]
    volatile unsigned int TIM1; 	      // STM[35:4]
    volatile unsigned int TIM2; 	      // STM[39:8]
    volatile unsigned int TIM3; 	      // STM[43:12]
    volatile unsigned int TIM4; 	      // STM[47:16]
    volatile unsigned int TIM5; 	      // STM[51:20]
    volatile unsigned int TIM6; 	      // STM[63:32]
    volatile unsigned int CAP; 		      // STM[63:32]
    volatile unsigned int CMP0;  		  // Compare Register 0
    volatile unsigned int CMP1;  		  // Compare Register 1
    volatile unsigned int CMCON; 	      // Compare Match Control Register
    volatile unsigned int ICR_;		      // Flags Clear Register
    volatile unsigned int ISCR;			  // Flags Enable Register
    volatile unsigned int RESERVED2[3];   // reserved (0x44 - 0x4C)
    volatile unsigned int TIM0SV;         // Timer 0 Register Second View
    volatile unsigned int CAPSV;          // Capture Register Second View
    volatile unsigned int RESERVED3[2];   // reserved (0x58, 0x5C)
    volatile unsigned int RESERVED4[9*4]; // reserved (0x60 ... 0xD0)
    volatile unsigned int RESERVED5[2];   // reserved (0xE0, 0xE4)
    volatile unsigned int OCS;            // OCDS Control and Status Register
    volatile unsigned int KRSTCLR;        // Reset Status Clear Register 12
    volatile unsigned int KRST1;          // Kernel Reset Control Register 1
    volatile unsigned int KRST0;          // Kernel Reset Control Register 0
    volatile unsigned int ACCEN1;         // ASCLIN Access Enable Register 1
    volatile unsigned int ACCEN0;         // ASCLIN Access Enable Register 0
} STM_t;

/***********************************************************
 * STM Initialization Structure
 ***********************************************************/
typedef struct
{
	uint32_t cmcon0;
	uint32_t cmcon1;
	uint32_t cmp0Icr;
	uint32_t cmp1Icr;
	uint32_t compVal0;
	uint32_t compVal1;
	uint32_t isr0;
	uint32_t isr1;
} STMInitStruct_t;

void stmInit(STM_t* const pSTM, const STMInitStruct_t* const pInit);

#endif /* STM_H_ */
